# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.
#
# Author:  Philippe Normand <philippe@fluendo.com>

from elisa.core import common
from elisa.core.utils.i18n import install_translation
from elisa.core.utils.i18n import get_current_locale
from elisa.core import log


from elisa.plugins.poblesec.base.hierarchy import HierarchyController
from elisa.plugins.poblesec.base.list import GenericListViewMode
from elisa.plugins.poblesec.base.preview_list import \
    MenuItemPreviewListController, MenuItemListController, \
    DoubleLineMenuItemPreviewListController

from elisa.plugins.poblesec.actions import Action, LinkAction

from elisa.plugins.pigment.graph.text import Text

from elisa.plugins.database import video_parser
from elisa.plugins.poblesec.simple_search_controller import SimpleSearchController
from elisa.plugins.database.video_controller import _get_thumbnail
from elisa.plugins.database import actions as database_actions
from elisa.plugins.database import models
from elisa.plugins.favorites.models import FavoritesItem

from elisa.plugins.thetvdb import utils as tvdb_utils
from elisa.plugins.thetvdb import models as thetvdb_models
from elisa.plugins.themoviedb import utils as themoviedb_utils

from twisted.internet import defer

from storm.tracer import debug

import sys
import os
import cgi

_ = install_translation('database')

class ControllerWithHeader(HierarchyController):
    """ Base class defining 2 new list header widgets:

    - one header text
    - one sub-header text, slightly smaller than the header
    """

    def initialize(self):
        dfr = super(ControllerWithHeader, self).initialize()
        self._header_text = Text()
        self._subheader_text = Text()
        self._header_text.markup = self.header()
        self._subheader_text.markup = self.subheader()
        return dfr

    def header(self):
        return ''

    def subheader(self):
        return ''

    def set_frontend(self, frontend):
        super(ControllerWithHeader, self).set_frontend(frontend)
        self.widget.add(self._header_text)
        self.widget.add(self._subheader_text)
        self._header_text.position = (0.05, 0.1, 0.)
        self._header_text.size = (1, 0.04)
        self._header_text.bg_a = 0
        self._header_text.visible = True
        self.nodes.y += 0.09
        self.nodes.height -= 0.09

        if self._subheader_text.markup:
            self._subheader_text.position = (0.05, 0.18, 0.)
            self._subheader_text.size = (1, 0.04)
            self._subheader_text.bg_a = 0
            self._subheader_text.visible = True
            self.nodes.y += 0.1

    def clean(self):
        self._header_text.clean()
        self._header_text = None
        self._subheader_text.clean()
        self._subheader_text = None
        return super(ControllerWithHeader, self).clean()

class CategoryController(ControllerWithHeader):
    """
    Entry point of the category changer chain of controllers. This
    controller displays for one video 3 actions allowing the user to
    change the category of a video to:

    - movie
    - tv show
    - unorganized video

    """

    def initialize(self, file_path=None, notifiers=None):
        self.file_path = file_path
        self.notifiers = notifiers
        dfr = super(CategoryController, self).initialize()
        self.actions.extend(self.make_actions())
        return dfr

    def header(self):
        filename = os.path.basename(self.file_path)
        return _("<b>Select the Category for:</b> %s") % cgi.escape(filename)

    def node_clicked(self, widget, item):
        dfr = item.run()

    def make_actions(self):
        actions = []

        action = database_actions.ToMovieAction(self)
        action.title = _('Movies')
        action.icon = 'elisa.plugins.poblesec.video_section'
        actions.append(action)

        action = database_actions.ToTVShowAction(self)
        action.title = _('TV Shows')
        action.icon = 'elisa.plugins.poblesec.tvshow'
        actions.append(action)

        action = LinkAction(self)
        action.title = _('Unorganized Videos')
        action.icon = 'elisa.plugins.poblesec.unorganized_videos'
        action.path = '/poblesec/database/recategorize/confirm/unclassified'
        action.args = {'file_path': self.file_path, 'notifiers': self.notifiers}
        actions.append(action)

        return actions

class ActionViewMode(GenericListViewMode):
    """ Generic view mode to display actions """

    def get_label(self, item):
        return defer.succeed(item.title)

    def get_default_image(self, item):
        return item.icon

    def get_image(self, item, theme):
        return None

    def get_preview_image(self, item, theme):
        return None

class CategoryVerticalWithPreview(CategoryController,
                                  MenuItemPreviewListController):
    view_mode = ActionViewMode


class TVShowRecategorizerController(ControllerWithHeader):
    """ Changing any video to a TV Show leads to this controller.
    """

    def initialize(self, file_path=None, series=None, search_term=None,
                   notifiers=None):
        self.file_path = file_path
        self.series = series
        self.search_term = search_term
        self.notifiers = notifiers

        def append_extra_actions():
            new_header_text = _('Please refine your query for the file below. Elisa will try to find some information.')
            new_search = database_actions.GoBackAction(self, new_header_text)
            new_search.title = _("Refine the search query")
            new_search.icon = 'elisa.plugins.poblesec.refine_search'
            self.model.append(new_search)
            manual = database_actions.ManualRecategorizeAction(self)
            manual.title = _("Manually enter the TV Show name")
            self.model.append(manual)

        def parent_initialize(result):
            return super(TVShowRecategorizerController, self).initialize()

        if series:
            self.model.extend(series)
        append_extra_actions()
        return parent_initialize(None)

    def header(self):
        filename = os.path.basename(self.file_path)
        if self.series:
            text = _("Select the TV Show for %s")
        else:
            text = _("No match found for %s")
        return text % cgi.escape(filename)

    def node_clicked(self, widget, item):
        if isinstance(item, Action):
            dfr = item.run()
        else:
            frontend = self.frontend
            controllers = frontend.retrieve_controllers('/poblesec/browser')
            history = controllers[0].history

            controller_path = '/poblesec/database/recategorize/tvshow/season'
            dfr = history.append_controller(controller_path, _("Seasons"),
                                            file_path=self.file_path,
                                            tvshow=item, notifiers=self.notifiers)
        return dfr


class TvShowViewMode(GenericListViewMode):
    """ View mode to display
    L{elisa.plugins.thetvdb.models.TvSeriesModel} models or actions.
    """

    def get_label(self, item):
        if isinstance(item, Action):
            return defer.succeed(item.title)
        else:
            return defer.succeed(item.name)

    def get_default_image(self, item):
        if isinstance(item, Action):
            resource = item.icon
        else:
            resource = 'elisa.plugins.poblesec.tvshow'
        return resource

    def get_image(self, item, theme):
        if isinstance(item, Action):
            return None
        if getattr(item, 'thumbnail_path', None) is not None:
            return defer.succeed(item.thumbnail_path)
        else:
            def got_banners(model):
                if model and model.season_banners:
                    seasons = model.season_banners.keys()
                    url = model.season_banners[seasons[0]].values()[0][0]
                    return _get_thumbnail(item, url)
            resource_manager = common.application.resource_manager
            url = item.season_banners_url
            model, dfr = resource_manager.get(url, item)
            dfr.addCallback(got_banners)
            return dfr

    def get_preview_image(self, item, theme):
        if isinstance(item, Action):
            return None
        if getattr(item, 'thumbnail_path', None) is not None:
            return item.thumbnail_path
        else:
            return None

class TVShowRecategorizerVerticalWithPreview(TVShowRecategorizerController,
                                             MenuItemPreviewListController):
    view_mode = TvShowViewMode


class TVSeasonRecategorizerController(ControllerWithHeader):
    """ If user selected a TV Show, this controller is then created,
    allowing the user to select a TV season for the show.
    """

    def initialize(self, file_path=None, tvshow=None, notifiers=None):
        self.file_path = file_path
        self.tvshow = tvshow
        self.notifiers = notifiers

        def got_seasons(seasons_model):
            seasons = seasons_model.seasons.values()
            seasons.sort(key=lambda season: season.number)
            for season in seasons:
                season.poster_urls = self.tvshow.season_banners.get(season.number, {})
            self.model.extend(seasons)
            return self

        def initialized(result):
            lang_code = unicode(get_current_locale().split('_')[0])
            lang_code = lang_code.lower()
            lookup_url = tvshow.all_episodes_url(lang_code)
            seasons_model = thetvdb_models.TvSeasonsListModel()
            model, dfr = common.application.resource_manager.get(lookup_url,
                                                                 seasons_model)
            dfr.addCallback(got_seasons)
            return dfr

        dfr = super(TVSeasonRecategorizerController, self).initialize()
        dfr.addCallback(initialized)
        return dfr

    def header(self):
        filename = os.path.basename(self.file_path)
        return _("Select the Season for %s") % cgi.escape(filename)

    def node_clicked(self, widget, item):
        if isinstance(item, Action):
            dfr = item.run()
        else:
            frontend = self.frontend
            controllers = frontend.retrieve_controllers('/poblesec/browser')
            history = controllers[0].history

            controller_path = '/poblesec/database/recategorize/tvshow/season/episode'
            dfr = history.append_controller(controller_path, _("Episode"),
                                            file_path=self.file_path,
                                            season=item,
                                            tvshow=self.tvshow,
                                            notifiers=self.notifiers)
        return dfr


class TvSeasonViewMode(GenericListViewMode):
    """ Generic view mode to display TV seasons """

    def get_label(self, item):
        return defer.succeed(item.tvshow_name)

    def get_sublabel(self, item):
        text = _("Season %s") % item.number
        return defer.succeed(text)

    def get_default_image(self, item):
        resource = 'elisa.plugins.poblesec.tvshow'
        return resource

    def get_image(self, item, theme):
        if getattr(item, 'thumbnail_path', None) is not None:
            return defer.succeed(item.thumbnail_path)
        else:
            lang_code = unicode(get_current_locale().split('_')[0])
            lang_code = lang_code.lower()
            poster_urls = item.poster_urls.get(lang_code, [])
            if not poster_urls:
                poster_urls = item.poster_urls.get(u'en', [])
            if poster_urls:
                return _get_thumbnail(item, poster_urls[0])
            else:
                resource = 'elisa.plugins.poblesec.tvshow'
                return defer.succeed(theme.get_resource(resource))

    def get_preview_image(self, item, theme):
        if getattr(item, 'thumbnail_path', None) is not None:
            return item.thumbnail_path
        else:
            return None

class TVSeasonRecategorizerVerticalWithPreview(TVSeasonRecategorizerController,
                                               DoubleLineMenuItemPreviewListController):
    view_mode = TvSeasonViewMode

class TVEpisodeRecategorizerController(ControllerWithHeader):
    """ If user selected a TV Show and a TV season this controller is
    created to allow him to select one Episode of the season.
    """

    def initialize(self, file_path=None, season=None, tvshow=None, notifiers=None):
        self.file_path = file_path
        self.season = season
        self.tvshow = tvshow
        self.notifiers = notifiers
        self.model.extend(self.season.episodes)
        return super(TVEpisodeRecategorizerController, self).initialize()

    def header(self):
        filename = os.path.basename(self.file_path)
        return _("Select the Episode for %s") % cgi.escape(filename)

    def node_clicked(self, widget, item):
        if isinstance(item, Action):
            dfr = item.run()
        else:
            frontend = self.frontend
            controllers = frontend.retrieve_controllers('/poblesec/browser')
            history = controllers[0].history

            controller_path = '/poblesec/database/recategorize/confirm/tvshow'
            dfr = history.append_controller(controller_path, _("Confirm"),
                                            file_path=self.file_path,
                                            tvshow=self.tvshow,
                                            season=self.season,
                                            episode=item,
                                            notifiers=self.notifiers)
        return dfr


class TvEpisodeViewMode(GenericListViewMode):
    """ Generic view mode for
    L{elisa.plugins.thetvdb.models.TvEpisodeModel} models
    """

    def get_label(self, item):
        text = _("Episode %s") % item.episode_number
        return defer.succeed(text)

    def get_sublabel(self, item):
        return defer.succeed(item.name)

    def get_default_image(self, item):
        resource = 'elisa.plugins.poblesec.tvshow'
        return resource

    def get_image(self, item, theme):
        if getattr(item, 'thumbnail_path', None) is not None:
            return defer.succeed(item.thumbnail_path)
        if item.poster_url:
            return _get_thumbnail(item, item.poster_url)

    def get_preview_image(self, item, theme):
        if getattr(item, 'thumbnail_path', None) is not None:
            return item.thumbnail_path
        else:
            return None

class TVEpisodeRecategorizerVerticalWithPreview(TVEpisodeRecategorizerController,
                                                DoubleLineMenuItemPreviewListController):
    view_mode = TvEpisodeViewMode


class TVShowSearchController(SimpleSearchController):
    """ This controller is created if no TV Show was initially
    automatically found for the video being recategorized. This
    controller presents a search entry and a OSK. Search is performed
    using thetvdb API.
    """


    def initialize(self, file_path=None, notifiers=None):
        self.file_path = file_path
        self.notifiers = notifiers
        title_text = _('No matches found. Please enter a new name for the file below. Elisa will try to find some information for it:')
        subtitle_text = os.path.basename(file_path)
        return super(TVShowSearchController, self).initialize(None,
                                                              title_text=title_text,
                                                              subtitle_text=subtitle_text)

    def do_search(self, search_term, history):
        lookup_url = tvdb_utils.get_series_lookup_url(search_term)
        model, dfr = common.application.resource_manager.get(lookup_url)

        def got_results(tvdb_model):
            controller_path = '/poblesec/database/recategorize/tvshow'
            args = {'file_path': self.file_path, 'search_term': search_term,
                    'notifiers': self.notifiers}
            if tvdb_model and tvdb_model.series:
                series = tvdb_model.series
                args['series'] = series
            else:
                args['series'] = []
            dfr = history.append_controller(controller_path,
                                            _("TV Shows"), **args)
            return dfr

        dfr.addCallback(got_results)
        return dfr


class ConfirmController(ControllerWithHeader, MenuItemListController):
    """ Generic controller presenting 2 actions, one to confirm the
    recategorization and one to cancel it. The controller has a simple
    API so that classes inheriting can implement recategorization
    scenarios (either the automatic one of the manual one).
    """

    view_mode = ActionViewMode

    def initialize(self, notifiers=None):
        self.notifiers = notifiers
        self.model.extend(self.make_actions())
        return super(ConfirmController, self).initialize()

    def node_clicked(self, widget, item):
        dfr = item.run()

    def make_actions(self):
        actions = []

        action = database_actions.ConfirmCategoryChangeAction(self)
        action.title = _('Confirm and Continue Browsing')
        action.icon = 'elisa.plugins.poblesec.enabled'
        action.confirm = self.confirm
        actions.append(action)

        action = database_actions.CancelCategoryChangeAction(self)
        action.title = _('Cancel')
        action.icon = 'elisa.plugins.poblesec.remove'
        actions.append(action)

        return actions

    def confirm(self, action):
        store = common.application.store
        file_path = self.file_path

        if log.getCategoryLevel(self.logCategory) > log.INFO:
            debug(True, stream=sys.stderr)

        def got_video(video, results):
            if video:
                if isinstance(video, models.Movie):
                    dfr = models.delete_movie(video, store)
                else:
                    dfr = models.delete_tvepisode(video, store)
                dfr.addCallback(commit)
                return dfr

        def got_result_set(results):
            dfr = results.one()
            dfr.addCallback(got_video, results)
            return dfr

        def find_old_episode(result):
            dfr = store.find(models.TVEpisode,
                             models.TVEpisode.file_path == file_path)
            dfr.addCallback(got_result_set)
            return dfr

        def add_in_db(result):
            if self.manual_add:
                dfr = self._manual_add()
            else:
                dfr = self._auto_add()
            return dfr

        def commit(result, final=False):
            if final:
                debug(False)
                for handler_id, notifier in self.notifiers:
                    notifier.emit('model-refresh-needed')

            dfr = store.commit()
            dfr.addCallback(lambda r: store.flush())
            dfr.addCallback(lambda r: result)
            return dfr

        def update_favorites(movie_or_episode):

            def got_videos(videos):
                for video in videos:
                    video.foreign_class = self.model_class_name
                    self._update_favorite_item(movie_or_episode, video)
                return store.commit()

            def fetch_videos(result_set):
                return result_set.all()

            dfr = store.find(FavoritesItem,
                             FavoritesItem.foreign_id == file_path)
            dfr.addCallback(fetch_videos)
            dfr.addCallback(got_videos)
            return dfr

        dfr = store.find(models.Movie, models.Movie.file_path == file_path)
        dfr.addCallback(got_result_set)
        dfr.addCallback(find_old_episode)
        dfr.addCallback(add_in_db)
        dfr.addCallback(update_favorites)
        dfr.addCallback(commit, True)
        return dfr

    def _auto_add(self):
        raise NotImplementedError()

    def _manual_add(self):
        raise NotImplementedError()

    def _update_favorite_item(self, new_db_item, favorites_item):
        raise NotImplementedError()

class TVShowConfirmController(ConfirmController):
    """ Confirm the choice of category change from any video to a TV
    Show episode.
    """
    model_class_name = u'TVEpisode'

    def initialize(self, file_path=None, tvshow=None, season=None,
                   tvshowid=None, tvshow_name=None, season_number=None,
                   episode=None, episode_number=None, episode_title=None,
                   notifiers=None):
        self.file_path = file_path
        self.episode = episode
        self.season = season
        self.tvshow = tvshow
        self.tvshow_name = tvshow_name
        if episode:
            self.episode_number = episode.episode_number
            self.episode_title = episode.name
            self.manual_add = False
        else:
            self.episode_number = episode_number
            self.episode_title = episode_title
            self.manual_add = True
        if season:
            self.season_number = season.number
        else:
            self.season_number = season_number
        if tvshow:
            self.tvshowid = tvshow.id
            self.tvshow_name = tvshow.name
        elif tvshowid:
            self.tvshowid = tvshowid
        else:
            self.tvshowid = None
        return super(TVShowConfirmController, self).initialize(notifiers=notifiers)

    def header(self):
        filename = cgi.escape(os.path.basename(self.file_path))
        return _("%s will now appear in TV Shows as:") % filename

    def subheader(self):
        return _("%s / Season %s / Episode %s") % (self.tvshow_name,
                                                   self.season_number,
                                                   self.episode_number)

    def confirm(self, action):
        store = common.application.store
        # check potential duplicate *before* modifying the db

        def duplicate_checked(is_dupe):
            if not is_dupe:
                return super(TVShowConfirmController, self).confirm(action)

        def got_episode(episode):
            if episode is None:
                return super(TVShowConfirmController, self).confirm(action)
            else:
                dfr = self._check_duplicate(episode)
                dfr.addCallback(duplicate_checked)
                return dfr

        def got_results(results):
            dfr = results.one()
            dfr.addCallback(got_episode)
            return dfr

        dfr = store.find(models.TVEpisode,
                         models.TVEpisode.file_path == self.file_path)
        dfr.addCallback(got_results)
        return dfr

    def _manual_add(self):
        store = common.application.store

        def commit(obj):
            dfr = store.commit()
            dfr.addCallback(lambda r: obj)
            return dfr

        def add_episode(season):
            episode = models.TVEpisode()
            episode.season_id = season.id
            episode.file_path = unicode(self.file_path)
            episode.number = int(self.episode_number)
            episode.name = unicode(self.episode_title)
            dfr = store.add(episode)
            dfr.addCallback(lambda result: episode)
            return dfr

        def add_season(tvshow):
            season = models.TVSeason()
            season.number = int(self.season_number)
            season.tvshow_id = tvshow.id
            dfr = store.add(season)
            dfr.addCallback(commit)
            dfr.addCallback(add_episode)
            return dfr

        def add_show():
            tvshow = models.TVShow()
            tvshow.name = unicode(self.tvshow_name)
            tvshow.filesystem_name = tvshow.name
            dfr = store.add(tvshow)
            dfr.addCallback(commit)
            dfr.addCallback(add_season)
            return dfr

        def episode_auto_added(episode):
            if episode is None:
                dfr = add_show()
            else:
                # video_parser managed to build (fetch in db) a model
                # representing that TVEpisode which means it's most
                # likely a duplicate.
                dfr = self._check_duplicate(episode)
                dfr.addCallback(lambda is_dupe: episode)
            return dfr

        parser = video_parser.VideoParser(store)
        dfr = parser._lookup_tv_episode(self.file_path,
                                        self.season_number,
                                        self.episode_number,
                                        self.tvshow_name)
        dfr.addCallback(episode_auto_added)
        return dfr

    def _check_duplicate(self, episode):

        def got_season(season, episode):
            dfr = season.tvshow
            dfr.addCallback(got_show, season, episode)
            return dfr

        def got_show(show, season, episode):
            is_dupe = (show.name == self.tvshow_name) and \
                      (season.number == int(self.season_number)) and \
                      (episode.number == int(self.episode_number))
            if is_dupe:
                # this episode is already in db
                frontend = self.frontend
                main = frontend.retrieve_controllers('/poblesec')[0]
                icon = 'elisa.plugins.poblesec.warning'
                title = _('You may have a duplicate')
                text = _('The video file %s is already categorized as %s, season %s, episode %s. Please double check the details.')
                text = text % (os.path.basename(episode.file_path),
                               self.tvshow_name,
                               self.season_number, self.episode_number)

                buttons = [(_('Ok'), main.hide_popup),]
                main.enqueue_popup(icon, title, text, buttons)
            return is_dupe

        dfr = episode.season
        dfr.addCallback(got_season, episode)
        return dfr

    def _auto_add(self):
        store = common.application.store

        def episode_auto_added(episode):
            if episode.file_path != self.file_path:
                # video_parser returned a model representing a video
                # file different from what we were recategorzing,
                # which means it's most likely a duplicate.
                dfr = self._check_duplicate(episode)
                dfr.addCallback(lambda is_dupe: episode)
                return dfr

        parser = video_parser.VideoParser(store)
        dfr = parser._lookup_tv_episode(self.file_path,
                                        self.season_number,
                                        self.episode_number,
                                        self.tvshow_name)
        dfr.addCallback(episode_auto_added)
        return dfr

    def _update_favorite_item(self, tvepisode, item):
        pass

class TVShowManualRecategorizer(SimpleSearchController):
    """ Allow the user to manually enter TV Show name, if he's not
    happy with thetvdb.com search results. 3 next screens then allow
    the user to enter season number, episode number and episode title.
    """

 
    def initialize(self, file_path=None, prefilled_text=None, notifiers=None):
        self.file_path = file_path
        self.notifiers = notifiers
        title_text = _('Unfortunately there is no available information. You can still manually organise this file. Confirm the title below or enter a new one')
        subtitle_text = os.path.basename(file_path)
        return super(TVShowManualRecategorizer, self).initialize(None,
                                                                 title_text=title_text,
                                                                 subtitle_text=subtitle_text,
                                                                 prefilled_text=prefilled_text)

    def do_search(self, search_term, history):
        controller_path = '/poblesec/database/recategorize/manual/tvshow/season'
        args = {'tvshow_name': search_term, 'file_path': self.file_path,
                'notifiers': self.notifiers}
        dfr = history.append_controller(controller_path,
                                        _("Enter Season"), **args)
        return dfr

class TVSeasonManualRecategorizer(SimpleSearchController):
    """ Allow the user to manually enter TV Season number, if he's not
    happy with thetvdb.com search results. 2 next screens then allow
    the user to enter episode number and episode title.
    """

    keyboard_layouts = ['elisa.plugins.poblesec.osk_numeric_special_chars',]

    def initialize(self, tvshow_name=None, file_path=None, notifiers=None):
        self.file_path = file_path
        self.tvshow_name = tvshow_name
        self.notifiers = notifiers
        title_text = _('Enter the season number for:')
        subtitle_text = os.path.basename(file_path)
        return super(TVSeasonManualRecategorizer, self).initialize(None,
                                                                   title_text=title_text,
                                                                   subtitle_text=subtitle_text)

    def do_search(self, search_term, history):
        controller_path = '/poblesec/database/recategorize/manual/tvshow/season/episode'
        args = {'tvshow_name': self.tvshow_name, 'notifiers': self.notifiers,
                'season_number': search_term, 'file_path': self.file_path}
        dfr = history.append_controller(controller_path,
                                        _("Enter Episode"), **args)
        return dfr

class TVEpisodeNumberManualRecategorizer(SimpleSearchController):
    """ Allow the user to manually enter TV Episode number, if he's
    not happy with thetvdb.com search results. Next screen then allows
    the user to enter episode title.
    """

    keyboard_layouts = ['elisa.plugins.poblesec.osk_numeric_special_chars',]

    def initialize(self, season_number=None, tvshow_name=None, file_path=None,
                   notifiers=None):
        self.file_path = file_path
        self.season_number = season_number
        self.tvshow_name = tvshow_name
        self.notifiers = notifiers
        title_text = _('Enter the episode number for:')
        subtitle_text = os.path.basename(file_path)
        return super(TVEpisodeNumberManualRecategorizer, self).initialize(None,
                                                                          title_text=title_text,
                                                                          subtitle_text=subtitle_text)

    def do_search(self, search_term, history):
        controller_path = '/poblesec/database/recategorize/manual/tvshow/season/episode/title'
        args = {'episode_number': search_term, 'file_path': self.file_path,
                'season_number': self.season_number,
                'tvshow_name': self.tvshow_name, 'notifiers': self.notifiers}
        dfr = history.append_controller(controller_path,
                                        _("Enter Episode title"), **args)
        return dfr

class TVEpisodeTitleManualRecategorizer(SimpleSearchController):
    """ Allow the user to manually enter TV Episode title, if he's not
    happy with thetvdb.com search results.
    """

    def initialize(self, season_number=None, tvshow_name=None,
                   episode_number=None, file_path=None, notifiers=None):
        self.file_path = file_path
        self.episode_number = episode_number
        self.season_number = season_number
        self.tvshow_name = tvshow_name
        self.notifiers = notifiers
        subtitle_text = os.path.basename(file_path)
        title_text = _('Enter the episode title for:')
        return super(TVEpisodeTitleManualRecategorizer, self).initialize(None,
                                                                         title_text=title_text,
                                                                         subtitle_text=subtitle_text)

    def do_search(self, search_term, history):
        controller_path = '/poblesec/database/recategorize/confirm/tvshow'
        args = {'episode_number': self.episode_number,
                'episode_title': search_term,
                'season_number': self.season_number,
                'tvshow_name': self.tvshow_name,
                'file_path': self.file_path, 'notifiers': self.notifiers}
        dfr = history.append_controller(controller_path,
                                        _("Confirm"), **args)
        return dfr

# Recategorization to Movies

class MovieRecategorizerController(ControllerWithHeader):
    """ Changing any video to a Movie leads to this controller.
    """

    def initialize(self, file_path=None, movies=None, search_term=None,
                   notifiers=None):
        self.file_path = file_path
        self.movies = movies
        self.search_term = search_term
        self.notifiers = notifiers

        if movies:
            self.model.extend(movies)

        new_header_text = _('Please refine your query for the file below. Elisa will try to find some information.')
        new_search = database_actions.GoBackAction(self, new_header_text)
        new_search.title = _("Refine the search query")
        new_search.icon = 'elisa.plugins.poblesec.refine_search'
        self.model.append(new_search)
        manual = database_actions.ManualRecategorizeAction(self)
        manual.title = _("Manually enter the movie name")
        manual.controller_path = '/poblesec/database/recategorize/manual/movie'
        manual.crumb_title = _("Enter Movie title")
        self.model.append(manual)
        return super(MovieRecategorizerController, self).initialize()

    def header(self):
        filename = os.path.basename(self.file_path)
        if self.movies:
            text = _("Select the Movie for %s")
        else:
            text = _("No match found for %s")
        return text % cgi.escape(filename)

    def node_clicked(self, widget, item):
        if isinstance(item, Action):
            dfr = item.run()
        else:
            frontend = self.frontend
            controllers = frontend.retrieve_controllers('/poblesec/browser')
            history = controllers[0].history

            controller_path = '/poblesec/database/recategorize/confirm/movie'
            dfr = history.append_controller(controller_path, _("Confirm"),
                                            file_path=self.file_path,
                                            movie=item, notifiers=self.notifiers)
        return dfr


class MovieViewMode(GenericListViewMode):
    """ Generic view mode for
    L{elisa.plugins.themoviedb.models.MovieModel} instances.
    """

    def get_label(self, item):
        return defer.succeed(item.title)

    def get_default_image(self, item):
        if isinstance(item, Action):
            resource = item.icon
        else:
            resource = 'elisa.plugins.poblesec.video_section'
        return resource

    def get_image(self, item, theme):
        if isinstance(item, Action):
            return None
        if getattr(item, 'thumbnail_path', None) is not None:
            return defer.succeed(item.thumbnail_path)
        else:
            def got_infos(model):
                urls = model.posters.get('mid',[])
                if urls:
                    url = urls[0]
                    return _get_thumbnail(item, url)
            resource_manager = common.application.resource_manager
            url = item.api_url
            model, dfr = resource_manager.get(url, item)
            dfr.addCallback(got_infos)
            return dfr

    def get_preview_image(self, item, theme):
        if isinstance(item, Action):
            return None
        if getattr(item, 'thumbnail_path', None) is not None:
            return item.thumbnail_path
        else:
            return None

class MovieRecategorizerVerticalWithPreview(MovieRecategorizerController,
                                            MenuItemPreviewListController):
    view_mode = MovieViewMode

class MovieSearchController(SimpleSearchController):
    """ This controller is created if no Movie was initially
    automatically found for the video being recategorized. This
    controller presents a search entry and a OSK. Search is performed
    using themoviedb API.
    """

    def initialize(self, file_path=None, notifiers=None):
        self.file_path = file_path
        self.notifiers = notifiers
        title_text = _('No matches found. Please enter a new name for:')
        subtitle_text = os.path.basename(file_path)
        return super(MovieSearchController, self).initialize(None,
                                                             title_text=title_text,
                                                             subtitle_text=subtitle_text)

    def do_search(self, search_term, history):
        lookup_url = themoviedb_utils.get_movie_lookup_url(search_term)
        model, dfr = common.application.resource_manager.get(lookup_url)

        def got_results(movies_model):
            return go_next(movies_model.movies)

        def go_next(movies):
            history.go_back()
            controller_path = '/poblesec/database/recategorize/movie'
            args = {'movies': movies, 'file_path': self.file_path,
                    'search_term': search_term, 'notifiers': self.notifiers}
            dfr = history.append_controller(controller_path,
                                            _("Change Category"), **args)
            return dfr

        def got_error(failure):
            return go_next(None)

        dfr.addCallback(got_results)
        dfr.addErrback(got_error)
        return dfr

class MovieConfirmController(ConfirmController):
    """ Confirm the choice of category change from any video to a
    Movie.
    """

    model_class_name = u'Movie'

    def initialize(self, file_path=None, movie=None, movie_title=None,
                   notifiers=None):
        self.file_path = file_path
        self.movie = movie
        if movie:
            self.movie_title = movie.title
            self.manual_add = False
        else:
            self.movie_title = movie_title
            self.manual_add = True
        return super(MovieConfirmController, self).initialize(notifiers)

    def header(self):
        filename = cgi.escape(os.path.basename(self.file_path))
        text = _("%s will now appear in Movies as:") % filename
        return text

    def subheader(self):
        return self.movie_title

    def confirm(self, action):
        store = common.application.store

        def got_movie(movie):
            if not self._check_duplicate(movie):
                return super(MovieConfirmController, self).confirm(action)

        def got_result_set(results):
            dfr = results.one()
            dfr.addCallback(got_movie)
            return dfr

        dfr = store.find(models.Movie, models.Movie.title == self.movie_title)
        dfr.addCallback(got_result_set)
        return dfr

    def _manual_add(self):
        store = common.application.store

        def got_movie(movie):
            if not movie:
                return manual_add()
            else:
                self._check_duplicate(movie)
                return movie

        def manual_add():
            movie = models.Movie()
            movie.title = self.movie_title
            movie.file_path = self.file_path
            dfr = store.add(movie)
            dfr.addCallback(lambda r: movie)
            return dfr

        parser = video_parser.VideoParser(store)
        dfr = parser._lookup_movie(self.file_path, self.movie_title)
        dfr.addCallback(got_movie)
        return dfr

    def _check_duplicate(self, movie):
        if movie and (movie.title == self.movie_title):
            # already in db
            frontend = self.frontend
            main = frontend.retrieve_controllers('/poblesec')[0]
            icon = 'elisa.plugins.poblesec.warning'
            title = _('You may have a duplicate')
            text = _('The video file %s is already categorized as %s. Please double check the details.')
            text = text % (os.path.basename(movie.file_path),
                           self.movie_title)

            buttons = [(_('Ok'), main.hide_popup),]
            main.enqueue_popup(icon, title, text, buttons)
            return True
        return False

    def _auto_add(self):
        store = common.application.store

        def commit(result):
            if result.file_path != self.file_path:
                self._check_duplicate(result)
            dfr = store.commit()
            dfr.addCallback(lambda r: result)
            return dfr

        parser = video_parser.VideoParser(store)
        dfr = parser._lookup_movie(self.file_path, self.movie_title,
                                   themoviedb_model=self.movie)
        dfr.addCallback(commit)
        return dfr

    def _update_favorite_item(self, movie, item):
        item.title = movie.title


class MovieManualRecategorizer(SimpleSearchController):
    """ Allow the user to manually enter Movie title, if he's not
    happy with themoviedb search results.
    """

    def initialize(self, file_path=None, prefilled_text=None, notifiers=None):
        self.file_path = file_path
        self.notifiers = notifiers
        title_text = _('Enter a new name for:')
        subtitle_text = os.path.basename(file_path)
        return super(MovieManualRecategorizer, self).initialize(None,
                                                                title_text=title_text,
                                                                subtitle_text=subtitle_text,
                                                                prefilled_text=prefilled_text)

    def do_search(self, search_term, history):
        controller_path = '/poblesec/database/recategorize/confirm/movie'
        args = {'movie_title': search_term, 'notifiers': self.notifiers,
                'file_path': self.file_path}
        dfr = history.append_controller(controller_path,
                                        _("Confirm"), **args)
        return dfr

class UnclassifiedConfirmController(ConfirmController):
    model_class_name = u'Video'

    def initialize(self, file_path=None, notifiers=None):
        self.manual_add = False
        self.file_path = file_path
        return super(UnclassifiedConfirmController, self).initialize(notifiers)

    def header(self):
        filename = cgi.escape(os.path.basename(self.file_path))
        text = _("%s will now appear in Unclassified videos") % filename
        return text

    def _manual_add(self):
        pass

    def _auto_add(self):
        pass

    def _update_favorite_item(self, video, item):
        pass
